@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_video.c,v 1.21 2000/12/05 20:03:45 mvojkovi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_video.c,v 1.20 2000/11/08 00:51:10 mvojkovi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -164,10 +164,6 @@ static XF86ImageRec Images[NUM_IMAGES] =
 	XVIMAGE_UYVY
 };
 
-#define outMGAdreg(reg, val) OUTREG8(RAMDAC_OFFSET + (reg), val)
-#define outMGAdac(reg, val) \
-        (outMGAdreg(MGA1064_INDEX, reg), outMGAdreg(MGA1064_DATA, val))
-
 static void 
 MGAResetVideoOverlay(ScrnInfoPtr pScrn) 
 {
